#include "moves.h"
#include "game_display.h"
#include "explode.h"
#include "options.h"
#include <stdlib.h>
#include <time.h>

ct_t
move_object_init(struct xor_move * move)
{
    move->to_x = move->from_x;
    move->to_y = move->from_y;
    switch (move->dir) {
    case MV_LEFT:
        move->to_x--;
        break;
    case MV_RIGHT:
        move->to_x++;
        break;
    case MV_UP:
        move->to_y--;
        break;
    case MV_DOWN:
        move->to_y++;
        break;
    default:
        return CT_ERROR;
    }
    move->from_obj = map->buf[move->from_y][move->from_x];
    move->to_obj = map->buf[move->to_y][move->to_x];
    ct_t cont = actions[move->to_obj].cont;

    /* ------------------------------------------ */
    /* conditions apply both to player and object */
    if (cont == CT_BLOCK || cont == CT_PASS)
        return cont;
    if (move->from_obj == ICON_DOLL)
        return CT_BLOCK;
    if (cont == CT_FILTER) {
        if (actions[move->to_obj].cont_dir & move->dir)
            return CT_PASS;
        return CT_BLOCK;
    }
    /* -------------------------------- */
    /* conditions which apply to player */
    if (actions[move->from_obj].mvini & MVI_PLAYER) {
        switch (cont) {
        case CT_EXIT:
        case CT_PICKUP:
        case CT_HARDPUSH:
        case CT_TELEPORT:
            return cont;
        default:
            if (cont & CT_PUSH) {
                if (actions[move->to_obj].cont_dir & move->dir)
                    return CT_PUSH;
                else
                    return CT_BLOCK;
            }
            return CT_ERROR;
        }
    }
    /* -------------------------------- */
    /* conditions which apply to object */
    if (cont & CT_EXPLODE)
        return CT_EXPLODE;
    return CT_BLOCK;
}

void
move_gravity_process(struct xor_move *xmv)
{
#ifdef DEBUG
    fprintf(stderr, "\nmove_gravity_process(xor_move* xmv=%lx)\n", (unsigned long) xmv);
#endif
    xmvlist_create();
    xmvlist_append_xor_move(xmv);
    xmvlist_first();
    struct timespec rpause;

    struct timespec repause;

    rpause.tv_sec = 0;
    if (player.replay)
        rpause.tv_nsec = options_replay_speed(options->replay_speed);
    else
        rpause.tv_nsec = 20000000L;
    while (xmvlist->current) {
        struct xor_move *xmv = xmvlist->current->xmv;

#ifdef DEBUG
        fprintf(stderr, "xmvlist->current=%lx->xmv=%lx\n",
               (unsigned long) xmvlist->current, (unsigned long) xmv);
#endif
        struct xor_move *cmv = xmv;

        while (cmv) {
            nanosleep(&rpause, &repause);
#ifdef DEBUG
            fprintf(stderr,
                "mv_grv_proc:\n\t*(%lx)cmv->from_x:%d,from_y:%d,from_obj:%s\n",
                (unsigned long) cmv, cmv->from_x, cmv->from_y,
                icons[cmv->from_obj].name);
#endif
            struct xor_move *cmv_next = (cmv->dir == MV_DOWN)
                ? cmv->up : cmv->right;

            ct_t cont = move_object_init(cmv);

            if (actions[cmv->to_obj].mvini == MVI_PLAYER) {
                if (cmv->moves_count > 0) {
                    player_death(cmv->to_obj);
                    cont = CT_PASS;
                }
            }
            if (cont == CT_EXPLODE) {
#ifdef DEBUG
                fprintf(stderr, "\nBANG!!!\n");
#endif
                explode_process_detonator(xmvlist->current);
                cmv = cmv_next;
#ifdef DEBUG
                fprintf(stderr,
                        "\nreturn to move_gravity_process(xor_move* xmv=%lx)\n",
                        (unsigned long) xmv);
                fprintf(stderr, "\tcmv=cmv_next=%lx\n", (unsigned long) cmv);
                if (cmv)
                    fprintf(stderr, "\t*(%lx)cmv->from_x:%d,from_y:%d,from_obj:%s\n",
                           (unsigned long) cmv, cmv->from_x, cmv->from_y,
                           icons[cmv->from_obj].name);
#endif
            }
            else if (cont == CT_PASS) {
                map->buf[cmv->from_y][cmv->from_x] = ICON_SPACE;
                map->buf[cmv->to_y][cmv->to_x] = cmv->from_obj;
                game_win_move_object(cmv);
                xy_t tmpx = cmv->from_x;

                xy_t tmpy = cmv->from_y;

                cmv->from_x = cmv->to_x;
                cmv->from_y = cmv->to_y;
                cmv->moves_count++;
                if (cmv_next) {
                    cmv = cmv_next;
                    if (!options->replay_hyper)
                        rpause.tv_nsec /= 1.1;
                }
                else {
                    if (cmv->dir == MV_DOWN)
                        cmv =
                            create_gravity_chain_xydir(tmpx + 1, tmpy, MV_LEFT);
                    else
                        cmv =
                            create_gravity_chain_xydir(tmpx, tmpy - 1, MV_DOWN);
                    if (cmv) {
                        xmvlist_append_xor_move(cmv);
                        if (!options->replay_hyper)
                            rpause.tv_nsec /= 1.1;
                    }
                    xmvlist_cycle_next();
                    cmv = xmv = xmvlist->current->xmv;
#ifdef DEBUG
                    fprintf(stderr, "xmvlist->current=%lx->xmv=%lx\n",
                           (unsigned long) xmvlist->current,
                           (unsigned long) xmv);
#endif
                }
            }
            else if ((cmv = move_unchain_blocked_bomb(xmvlist->current)))
                xmv = cmv;
            else {              /* chain blocked - remove it */
#ifdef DEBUG
                fprintf(stderr, "mv_grav_proc() chain blocked removing...\n");
#endif
                struct xmv_link *tmplnk = xmvlist->current;

#ifdef DEBUG
                if (tmplnk->xmv != xmv)
                    fprintf(stderr, "\n***** current->xmv != xmv *******\n");
#endif
                destroy_gravity_chain(xmv);
                if (xmvlist_cycle_next() == tmplnk)
                    cmv = xmv = 0;      /* about to remove last chain */
                else
                    cmv = xmv = xmvlist->current->xmv;
                xmvlist_unlink_xor_move(tmplnk);
            }
        }
    }
    xmvlist_destroy();
#ifdef DEBUG
    fprintf(stderr, "\nEXITING move_gravity_process(xor_move* xmv=%lx)\n",
           (unsigned long) xmv);
#endif
}

struct xor_move *
move_unchain_blocked_bomb(struct xmv_link *lnk)
{
    struct xor_move *xmv = lnk->xmv;

#ifdef DEBUG
    fprintf(stderr, "move_unchain_blocked_bomb(xmv_link* lnk=%lx)\n",
           (unsigned long) lnk);
    fprintf(stderr, "moves count:%d\n", xmv->moves_count);
#endif

    if (xmv->moves_count == 0)
        return 0;
    struct xor_move *tmp;

    switch (xmv->dir) {
    case MV_DOWN:
        while (xmv) {
            if (xmv->from_obj == ICON_H_BOMB) {
                if ((tmp = xmv->up)) {
                    xmv->up = 0;
                    destroy_gravity_chain(lnk->xmv);
                    return (lnk->xmv = tmp);
                }
                return 0;
            }
            else {
                tmp = xmv;
                xmv = xmv->up;
            }
        }
        return 0;
    case MV_LEFT:
        while (xmv) {
            if (xmv->from_obj == ICON_V_BOMB) {
                if ((tmp = xmv->right)) {
                    xmv->right = 0;
                    destroy_gravity_chain(lnk->xmv);
                    return (lnk->xmv = tmp);
                }
                return 0;
            }
            else {
                tmp = xmv;
                xmv = xmv->right;
            }
        }
        return 0;
    default:
        return 0;
    }
}

/*  thankfully, the doll cannot move through force-fields, nor
    detonate bombs. so all we need do is check that where it is
    moving is ICON_SPACE to continue moving it, otherwise stop.
*/
void
move_hard_push_doll(struct xor_move *xmv)
{
    struct timespec rpause;

    struct timespec repause;

    rpause.tv_sec = 0;
    if (player.replay)
        rpause.tv_nsec = options_replay_speed(options->replay_speed);
    else
        rpause.tv_nsec = 20000000L;
    while (map->buf[xmv->to_y][xmv->to_x] == ICON_SPACE) {
        map->buf[xmv->from_y][xmv->from_x] = ICON_SPACE;
        map->buf[xmv->to_y][xmv->to_x] = ICON_DOLL;
        game_win_move_object(xmv);
        xmv->from_x = xmv->to_x;
        xmv->from_y = xmv->to_y;
        switch (xmv->dir) {
        case MV_LEFT:
            xmv->to_x--;
            break;
        case MV_RIGHT:
            xmv->to_x++;
            break;
        case MV_UP:
            xmv->to_y--;
            break;
        case MV_DOWN:
            xmv->to_y++;
            break;
        default:
            return;
        }
        nanosleep(&rpause, &repause);
    }
    free(xmv);
}
